//+--------------------------------------------------------------------+
//|                                                 mt4_dataseries.mqh |
//| MT4-ish time series functions custom implementation for MT5 in MQL |
//| Not needed after build 1860, where native implementation is added  |
//+--------------------------------------------------------------------+


int iBarShiftFast(string symbol, ENUM_TIMEFRAMES timeframe, datetime t)
{
  return(Bars(symbol, timeframe, t, D'2121.01.01') - 1);
}

int iBarShift(string symbol, ENUM_TIMEFRAMES timeframe, datetime t, bool Exact = true)
{
  datetime lastBar;
  SeriesInfoInteger(symbol, timeframe, SERIES_LASTBAR_DATE, lastBar);
  return(Bars(symbol, timeframe, t, lastBar) - 1);
}


#define DefineCopy(NAME,TYPE) \
TYPE i##NAME(string symbol, ENUM_TIMEFRAMES tf, int b) \
{ \
  TYPE result[1]; \
  return Copy##NAME(symbol, tf, b, 1, result) > 0 ? result[0] : 0; \
}

DefineCopy(Time,datetime);
DefineCopy(High,double);
DefineCopy(Low,double);
DefineCopy(Open,double);
DefineCopy(Close,double);

long iVolume(string symbol, ENUM_TIMEFRAMES tf, int b)
{
  long result[1];
  return CopyTickVolume(symbol, tf, b, 1, result) > 0 ? result[0] : 0;
}

#define MODE_OPEN   0
#define MODE_LOW    1
#define MODE_HIGH   2
#define MODE_CLOSE  3
#define MODE_VOLUME 4
#define MODE_TIME   5

int iHighest(string symbol, ENUM_TIMEFRAMES tf, int type, int count = WHOLE_ARRAY, int b = 0)
{
  double data[];
  long v[];
  ArraySetAsSeries(data, true);
  ArraySetAsSeries(v, true);
  switch(type)
  {
    case MODE_OPEN:
      if(CopyOpen(symbol, tf, b, count, data) > -1)
      {
        return ArrayMaximum(data, 0, count) + b;
      }
      break;
    case MODE_LOW:
      if(CopyLow(symbol, tf, b, count, data) > -1)
      {
        return ArrayMaximum(data, 0, count) + b;
      }
      break;
    case MODE_HIGH:
      if(CopyHigh(symbol, tf, b, count, data) > -1)
      {
        return ArrayMaximum(data, 0, count) + b;
      }
      break;
    case MODE_CLOSE:
      if(CopyClose(symbol, tf, b, count, data) > -1)
      {
        return ArrayMaximum(data, 0, count) + b;
      }
      break;
    case MODE_VOLUME:
      if(CopyTickVolume(symbol, tf, b, count, v) > -1)
      {
        return ArrayMaximum(v, 0, count) + b;
      }
      break;
  }
  return -1;
}

int iLowest(string symbol, ENUM_TIMEFRAMES tf, int type, int count = WHOLE_ARRAY, int b = 0)
{
  double data[];
  long v[];
  ArraySetAsSeries(data, true);
  ArraySetAsSeries(v, true);
  switch(type)
  {
    case MODE_OPEN:
      if(CopyOpen(symbol, tf, b, count, data) > -1)
      {
        return ArrayMinimum(data, 0, count) + b;
      }
      break;
    case MODE_LOW:
      if(CopyLow(symbol, tf, b, count, data) > -1)
      {
        return ArrayMinimum(data, 0, count) + b;
      }
      break;
    case MODE_HIGH:
      if(CopyHigh(symbol, tf, b, count, data) > -1)
      {
        return ArrayMinimum(data, 0, count) + b;
      }
      break;
    case MODE_CLOSE:
      if(CopyClose(symbol, tf, b, count, data) > -1)
      {
        return ArrayMinimum(data, 0, count) + b;
      }
      break;
    case MODE_VOLUME:
      if(CopyTickVolume(symbol, tf, b, count, v) > -1)
      {
        return ArrayMinimum(v, 0, count) + b;
      }
      break;
  }
  return -1;
}

/* EXAMPLE
class OpenBroker
{
  public:
    double operator[](int b)
    {
      return iOpen(_Symbol, _Period, b);
    }
};
OpenBroker Open;
*/

#define DefineBroker(NAME,TYPE) \
class NAME##Broker \
{ \
  public: \
    TYPE operator[](int b) \
    { \
      return i##NAME(_Symbol, _Period, b); \
    } \
}; \
NAME##Broker NAME;

DefineBroker(Time, datetime);
DefineBroker(Open, double);
DefineBroker(High, double);
DefineBroker(Low, double);
DefineBroker(Close, double);
DefineBroker(Volume, long);
